@@ -264,9 +264,13 @@ class nnUNetTrainerV2(nnUNetTrainer):
 
     def do_split(self):
         """
-        we now allow more than 5 splits. IMPORTANT: and fold > 4 will not be a real split but just another random
-        80:20 split of the data. You cannot run X-fold cross-validation with this code. It will always be a 5-fold CV.
-        Folds > 4 will be independent from each other
+        The default split is a 5 fold CV on all available training cases. nnU-Net will create a split (it is seeded,
+        so always the same) and save it as splits_final.pkl file in the preprocessed data directory.
+        Sometimes you may want to create your own split for various reasons. For this you will need to create your own
+        splits_final.pkl file. If this file is present, nnU-Net is going to use it and whatever splits are defined in
+        it. You can create as many splits in this file as you want. Note that if you define only 4 splits (fold 0-3)
+        and then set fold=4 when training (that would be the fifth split), nnU-Net will print a warning and proceed to
+        use a random 80:20 data split.
         :return:
         """
         if self.fold == "all":
