@@ -223,7 +223,6 @@ class nnUNetTrainerV2_5epochs_noDS(nnUNetTrainerV2_noDeepSupervision):
         pass
 
 
-
 class nnUNetTrainerV2_DDP_5epochs(nnUNetTrainerV2_DDP):
     def __init__(self, plans_file, fold, local_rank, output_folder=None, dataset_directory=None, batch_dice=True,
                  stage=None,
@@ -247,4 +246,48 @@ class nnUNetTrainerV2_DDP_5epochs(nnUNetTrainerV2_DDP):
         pass
 
     def save_checkpoint(self, fname, save_optimizer=True):
-        pass
\ No newline at end of file
+        pass
+
+
+class nnUNetTrainerV2_DDP_5epochs_dummyLoad(nnUNetTrainerV2_DDP_5epochs):
+    def initialize(self, training=True, force_load_plans=False):
+        super().initialize(training, force_load_plans)
+        self.some_batch = torch.rand((self.batch_size, self.num_input_channels, *self.patch_size)).float().cuda()
+
+        self.some_gt = [torch.round(torch.rand((self.batch_size, 1, *[int(i * j) for i, j in zip(self.patch_size, k)])) * (
+                    self.num_classes - 1)).float().cuda() for k in self.deep_supervision_scales]
+
+    def run_iteration(self, data_generator, do_backprop=True, run_online_evaluation=False):
+        data = self.some_batch
+        target = self.some_gt
+
+        self.optimizer.zero_grad()
+
+        if self.fp16:
+            with autocast():
+                output = self.network(data)
+                del data
+                l = self.compute_loss(output, target)
+
+            if do_backprop:
+                self.amp_grad_scaler.scale(l).backward()
+                self.amp_grad_scaler.unscale_(self.optimizer)
+                torch.nn.utils.clip_grad_norm_(self.network.parameters(), 12)
+                self.amp_grad_scaler.step(self.optimizer)
+                self.amp_grad_scaler.update()
+        else:
+            output = self.network(data)
+            del data
+            l = self.compute_loss(output, target)
+
+            if do_backprop:
+                l.backward()
+                torch.nn.utils.clip_grad_norm_(self.network.parameters(), 12)
+                self.optimizer.step()
+
+        if run_online_evaluation:
+            self.run_online_evaluation(output, target)
+
+        del target
+
+        return l.detach().cpu().numpy()
\ No newline at end of file
